"{ Package: 'jv:tea/compiler' }"

"{ NameSpace: Smalltalk }"

RBParser subclass:#TParser
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Languages-Tea-Compiler-AST'
!

!TParser class methodsFor:'parsing'!

parseMethod: aString 
    ^ self parseMethod: aString onError: [:msg :pos | self error: msg , ' at ', pos printString ]

    "Created: / 20-08-2015 / 17:04:49 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!TParser methodsFor:'private-parsing'!

parseArgOrLocal
    "Parse either method/block argument or a local (inside | | )"

    | variable |

    variable := self parseVariableNode.
    variable typeSpec: (self parseTypeSpec: false).
    ^ variable

    "Created: / 20-08-2015 / 16:57:55 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 21-08-2015 / 21:20:13 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

parseBinaryPattern
    | method |

    method := super parseBinaryPattern.
    method returnTypeSpec: (self parseTypeSpec: true).
    ^ method

    "Created: / 21-08-2015 / 22:55:43 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

parseKeywordPattern
    | method |

    method := super parseKeywordPattern.
    method returnTypeSpec: (self parseTypeSpec: true).
    ^ method

    "Created: / 20-08-2015 / 17:33:09 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 21-08-2015 / 22:50:23 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

parseType
    "
    type ::= type_simple ( '|' type )*
    "

    | types |
    types := OrderedCollection with: self parseTypeSimple.
    [ currentToken isBinary and:[ currentToken value == #| ] ] whileTrue:[ 

        self step. "/ eat bar.
        types add: self parseTypeSimple.
    ].
    ^ types size == 1 ifTrue:[ types anElement ] ifFalse:[ TUnionTypeNode new types: types ]

    "Created: / 20-08-2015 / 17:18:11 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 21-08-2015 / 21:12:33 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

parseTypeSimple
    "
    type_simple::= identifier ('[' type_parameters ']')?
    "
    | type |
        
    currentToken isIdentifier ifTrue:[ 
        type  := TSimpleTypeNode new.
        type name: currentToken value.
    ] ifFalse:[ 
        (currentToken isLiteral and:[ currentToken value isNil ]) ifFalse:[
            self parserError: 'type identifier expected'.
        ].
        type := TSimpleTypeNode new.
        type name: 'nil'.
    ].
    type
        start: currentToken start;
        stop: currentToken stop;
        lineNumber: currentToken lineNumber.  
    self step. "/ eat identifier.
    ^ type

    "Created: / 20-08-2015 / 17:20:16 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 21-08-2015 / 21:13:01 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

parseTypeSpec: forReturn
    (currentToken isBinary and: [currentToken value == #<]) ifTrue: [
        | start stop type |    
        start := currentToken start.
        self step.
        forReturn ifTrue:[ 
            (currentToken isSpecial and:[ currentToken value == $^ ]) ifFalse:[ 
                self parserError: '''^'' expected'.
            ].
            self step.
        ].          

        type := self parseType.

        (currentToken isBinary and: [currentToken value == #>]) 
                ifFalse: [self parserError: '''>'' expected'].
        stop := currentToken stop.
        self step.
        ^ TTypeSpecNode new
            type: type;
            start: start;
            stop: stop.
    ] ifFalse:[ 
        self parserError: 'type specification expected'
    ].

    "Created: / 20-08-2015 / 17:13:10 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 21-08-2015 / 21:18:41 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

parseUnaryPattern
    | method |

    method := super parseUnaryPattern.
    method returnTypeSpec: (self parseTypeSpec: true).
    ^ method

    "Created: / 21-08-2015 / 22:54:11 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

